"""
Description:
    A Module to log and store information about a Pokémon tcg collection
Usage:
    To run as a program "python3 main.py"
    To use as a library:
        "from pokemonCardLogger import clss_pickle as pcl" for pickle storage
    To get just the packs and their ids, run "python3 packRef.py"
"""
from pokemonCardLogger import main, test_api_status, clss_pickle, clss_base, assets, packRef, post_encrypt
